//--------------------------------------------------------------------
//
//  Implementation of Turtle functions             Turtle.cpp
//  by Mary Johnson   July 1997
//--------------------------------------------------------------------

#include <math.h>
//--------------------------------------------------------------------

#include "turtle.h"

// Constructors

   Turtle::Turtle (window & win, int xCoord, int yCoord, int head)
   : Color (BLACK), TWindow(win)
   {
      heading = head;
      pen =1;
      x = xCoord;
      y = yCoord;
      TWindow.SetWaitClose(true);
   }

   Turtle::Turtle (window & win, int xCoord, int yCoord)
   : Color (BLACK), TWindow(win)
   {
      heading = 90;
      pen =1;
      x = xCoord;
      y = yCoord;
      TWindow.SetWaitClose(true);
   }

   Turtle::Turtle (window & win)
   : Color (BLACK), TWindow(win)
   {
      pen = 1;
      home();
      TWindow.SetWaitClose(true);
   }

   Turtle::~Turtle()
   {}

//--------------------------------------------------------------------
   void Turtle::forward (int amt)// move the Turtle amt steps forward;
   {              // drawing a line in pencolor if the pen is down.
      TWindow.SetPen(Color);
      int dx = amt * cos(heading*cdPi/180);
      int dy = - (amt * sin (heading*cdPi/180));
      if ( pen)
         TWindow.DrawLine(x,y,x+dx,y+dy);
      x = x + dx;
      y = y + dy;
   }

//--------------------------------------------------------------------
   void Turtle::back (int amt)  // move the Turtle back amt steps; drawing a
   {                   // line in pencolor if the pen is down.
      forward(-amt);
   }

//--------------------------------------------------------------------
   void Turtle::turnRight (int amt)    // turn the Turtle to the right
   {                                     // amt degrees.
      heading = heading - amt;
   }

//--------------------------------------------------------------------
   void Turtle::turnLeft (int amt)    // turn the Turtle to the left
   {                                    // amt degrees.
      turnRight (-amt);
   }

//--------------------------------------------------------------------
   void Turtle::home () // move the Turtle to the home position - center of
   {               // the screen, facing north.
      x = TWindow.GetWidth()/2;            // x center
      y = TWindow.GetHeight()/2;            // y center
      heading = 90;               // heading
   }

//--------------------------------------------------------------------
   void Turtle::clear ()   // clear screen; position and direction of
   {                  // Turtle unchanged.
      TWindow.SetPen(WHITE, 0);
      TWindow.SetBrush(WHITE);
      TWindow.DrawRectangle(0, 0, TWindow.GetWidth(), TWindow.GetHeight());
   }

//--------------------------------------------------------------------
   void Turtle::clearAll ()// clear screen; move Turtle to home position;
   {
      TWindow.SetPen(WHITE, 0);
      TWindow.SetBrush(WHITE);
      TWindow.DrawRectangle(0, 0, TWindow.GetWidth(), TWindow.GetHeight());
      home();
   }

//--------------------------------------------------------------------
   void Turtle::penColor (color kolor) // change pen color to the
                                                  //indicated color
   {
      Color = kolor;
      TWindow.SetPen(Color);
   }

//------------------------------------------------------------------------
   void Turtle::penColor (int x)   // change pen color based on integer
   {
      x = abs(x) % 10;
      switch (x)
      {
         case 1:
            Color = BLACK;
            break;
         case 2:
            Color = GRAY;
            break;
         case 3:
            Color = BLUE;
            break;
         case 4:
            Color = RED;
            break;
         case 5:
            Color = GREEN;
            break;
         case 6:
            Color = YELLOW;
            break;
         case 7:
            Color = ORANGE;
            break;
         case 8:
            Color = CYAN;
            break;
         case 9:           
            Color = MAGENTA;
            break;
         case 10:
            Color = WHITE;
            break;
         default:
            Color = BLACK;
            break;
      }
      TWindow.SetPen(Color);
   }

//--------------------------------------------------------------------
   void Turtle::penUp ()           // pen up - no lines will be drawn.
   {
      pen = 0;
   }

//--------------------------------------------------------------------
   void Turtle::penDown ()         // pen down - lines will be drawn.
   {
      pen = 1;
   }

//--------------------------------------------------------------------
// C turtle graphics functions

static Turtle * cTurtle= 0;

// must be called before working with a turtle.
void createTurtle(window &win)
{
    if (cTurtle == 0)
        cTurtle = new Turtle(win);
}

// move the Turtle amt steps forward; drawing a
// line in pencolor if the pen is down.
void forward(int amt){ cTurtle->forward(amt);}

// move the Turtle back amt steps, drawing a line in pencolor 
// if the pen is down.
void back(int amt){ cTurtle->back(amt);}

// turn the Turtle to the right amt degrees.
void turnRight(int amt){ cTurtle->turnRight(amt); }

// turn the Turtle to the left amt degrees.
void turnLeft(int amt){ cTurtle->turnLeft(amt);}

// move Turtle to home position - center of screen, facing north.
void home(){ cTurtle->home();}

// clear screen; position and direction of Turtle unchanged.
void clear(){ cTurtle->clear(); }

// clear screen; move Turtle to home position.
void clearAll(){ cTurtle->clearAll();}

// change pen color to the color indicated.
void penColor (color kolor){ cTurtle->penColor(kolor);}

// change pen color:
// 1 - BLACK,  2 - GRAY,   3 - BLUE,   4 - RED,      5 - GREEN,
// 6 - YELLOW, 7 - ORANGE, 8 - CYAN,   9 - MAGENTA, 10 - WHITE.
void penColor (int icolor){ cTurtle->penColor(icolor);}

// pen up - no lines will be drawn.
void penUp (){ cTurtle->penUp();}

// pen down - lines will be drawn.
void penDown (){ cTurtle->penDown();}
